"""This script will check new package versions of either
your current installed distributions or a buildout file if provided.
It can detect major or minor versions availability:
level 0 gets the highest version (X.y.z),
level 1 gets the highest intermediate version (x.Y.z),
level 2 gets the highest minor version (x.y.Z).

Using level 2, you can automatically retrieve all bugfix versions of a buildout.
"""

from optparse import OptionParser

def main():

    parser = OptionParser(description=__doc__)

    parser.add_option('-l', '--level',
                      type='int',
                      dest='level',
                      default=0,
                      help=u"Version level to check")

    parser.add_option('-i', '--index',
                      dest='index',
                      help=u"Alternative package index URL")

    options, args = parser.parse_args()

    if len(args) > 1:
        parser.error("You must specify only one argument")

    buildoutcfg = False
    if len(args) == 1:
        buildoutcfg = args[0]

    kw = {}
    if options.index is not None:
        kw['index_url'] = options.index

    if buildoutcfg:
        import buildout
        checker = buildout.Checker(filename=buildoutcfg, **kw)
    else:
        import installed
        checker = installed.Checker()

    checker.check(level=options.level)


if __name__ == '__main__':
    main()




from doctest import DocFileSuite, ELLIPSIS, NORMALIZE_WHITESPACE
import unittest

def test():
    optionflags = ELLIPSIS|NORMALIZE_WHITESPACE
    suite = unittest.TestSuite()
    suite.addTest(DocFileSuite('README.txt', optionflags=optionflags))
    suite.addTest(DocFileSuite('buildout.txt', optionflags=optionflags))
    suite.addTest(DocFileSuite('installed.txt', optionflags=optionflags))

    return suite



from pkg_resources import parse_version, Requirement
from setuptools import package_index

class Checker(object):
    """Base class for version checkers
    """
    __custom_url = False
    def __init__(self, filename=None, index_url=None):
        self.filename = filename
        self.pi = package_index.PackageIndex()
        self._set_index_url(index_url)
        if index_url is not None:
            self.__custom_url = True

    def _set_index_url(self, url):
        """set the index URL
        """
        if url is not None:
            self.pi.index_url = url
        if not self.pi.index_url.endswith('/'):
            self.pi.index_url += '/'

    def check(self, level=0):
        """Search new versions in a version list
        versions must be a dict {'name': 'version'}

        The new version is limited to the given level:
        Example with version x.y.z
        level = 0: checks new version x
        level = 1: checks new version y
        level = 2: checks new version z

        By default, the highest version is found.
        """
        versions = self.get_versions()

        for name, version in versions.items():
            parsed_version = parse_version(version)
            req = Requirement.parse(name)
            self.pi.find_packages(req)
            new_dist = None
            # loop all versions until we find the first newer version
            # that keeps the major versions (below level)
            for dist in self.pi[req.key]:
                if dist.parsed_version[:level] > parsed_version[:level]:
                    continue
                new_dist = dist
                break


            if new_dist and new_dist.parsed_version > parsed_version:
                print("%s=%s" % (name, new_dist.version))


    def get_versions(self):
        raise NotImplementedError



import pkg_resources
from z3c.checkversions import base

class Checker(base.Checker):
    """Checker class for installed packages
    """
    def get_versions(self, level=0):
        working_set = pkg_resources.working_set
        versions = dict([(d.key, d.version) for d in working_set])
        print "Checking your installed distributions"
        return versions





from zc.buildout.buildout import Buildout
from z3c.checkversions import base


class Checker(base.Checker):
    """checker class for a buildout
    """
    def get_versions(self):
        buildout = Buildout(self.filename, '')

        # set the index URL from the buildout if not already provided
        buildout_index = buildout['buildout'].get('index')
        if not self.__custom_url:
            self._set_index_url(buildout_index)

        print(u"Checking buildout file %s" % self.filename)
        return buildout['versions']


